/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

#ifndef __VNCIPC_H__
#define __VNCIPC_H__

#include <e32base.h>

#include <sys\stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys\select.h>
#include <sys\sem.h>
#include <sys\stat.h>

// Some maximum lengths
#define KMaxPasswordLength 16
#define KMaxKeyLength 10

// The message type
enum TMsgType
	{
	EAcknowledge,				// Acknowledge a request
	EShowPasswordReq,			// Show the password request
	EShowPasswordResp,			// Send the password response
	ECreateBitmap,				// A bitmap has been created, duplicate
	EGraphicsExposeEvent,		// A Graphics expose event has been sent
	EPutImage,					// Put an image on the screen
	EMouseEvent,				// A mouse event has occurred
	EKeyEvent,					// A keypad event has occurred
	ECopyClipboardToVNCReq,		// Request to copy clipboard to VNC server	
	ECopyClipboardFromVNCReq,	// Request to copy clipboard from VNC server
	ECopyClipboardToVNCAck,		// Request to copy clipboard acknowledge
	ECopyClipboardTransfer,		// Transfer the clipboard contents
	ERefreshReq,				// Refresh the screen
	EMoveSoftCursor,			// Move the soft cursor
	ECreatePixmapCursorReq,		// Create a local cursor request
	ECreatePixmapCursorResp,	// Create local cursor response
	EFreeCursor,				// Free the cursor
	EWarpPointer,				// Warp the local cursor (move it)
	ESetFullscreen,				// Full screen mode on/off
	EQuitReq					// Quit the engine
	};

// Mouse messages
enum TMouseType
	{
	EMouse1Down,
	EMouse1Up,
	EMouse2Down,
	EMouse2Up,
	EMouse3Down,
	EMouse3Up,
	EMouseDrag,
	EMouseMove
	};

// Keyboard messages
enum TKeyType
	{
	EKeyDown,
	EKeyUp,
	EKey
	};
	
// Keyboard modifiers
enum TKeyModifier
	{
	EKeyModShift = 1,
	EKeyModCtrl = 2,
	EKeyModAlt = 4,
	EKeyModFunc = 8
	};
	
// The message
struct vncmsg
	{
	int iMsgType;							// The message type
	union 
		{
		char iPassword[KMaxPasswordLength];	// Password response data
		
		struct 
			{
			TInt iHandle;
			} iCreateBitmap;				// Create bitmap request

		struct 
			{
			TInt iX;
			TInt iY;
			TInt iWidth;
			TInt iHeight;
			} iGraphicsExposeEvent;			// Graphics expose event

		struct 
			{
			TInt iX;
			TInt iY;
			TInt iWidth;
			TInt iHeight;
			} iPutImage;					// Put an image on the display
	
		struct 
			{
			TInt iType;
			TInt iX;
			TInt iY;
			} iMouseEvent;					// Mouse event

		struct 
			{
			TInt iType;
			char iKeyChar[KMaxKeyLength];
			} iKeyEvent;					// Keypad event
			
		struct 
			{
			TInt iLength;
			} iCopyClipboard;				// Copy clipboard request

		struct 
			{
			TInt iX;
			TInt iY;
			} iMoveSoftCursor;				// Move a soft cursor

		struct 
			{
			TInt iPixmap;
			TInt iMask;
			TInt iHotPointX;
			TInt iHotPointY;
			} iCreatePixmapCursorReq;		// Create a cursor request

		struct 
			{
			TInt iCursor;
			} iCreatePixmapCursorResp;		// Cursor created response

		struct 
			{
			TInt iCursorX;
			TInt iCursorY;
			} iWarpPointer;					// Move the local cursor

		struct
			{
			TInt iCursor;
			} iFreeCursor;					// Free the local cursor

		struct
			{
			TBool iFullscreen;
			} iSetFullscreen;				// Fullscreen mode
		} iData;
	};
	
/*
 * Class to handle IPC between UI and engine
 */
class CVncIpc : public CBase
	{
public:
	// The pipe identifiers
	enum TWaitPipe
		{
		EWaitPipeNormal,
		EWaitPipeStdErr,
		EWaitPipeQuit
		};
		
	IMPORT_C static CVncIpc* NewL(TBool aServer);
	IMPORT_C virtual ~CVncIpc();

	// Is there an event waiting for me?
	IMPORT_C int EventFifo();

	// Wait for data
	IMPORT_C TWaitPipe WaitForData(int aStdErr);

	// Quit
	IMPORT_C void Quit();
	
	// Send an event
	IMPORT_C void SendEventL(TInt aMsgType, vncmsg& aMessage);
		
	// Send a message to the other side
	IMPORT_C void SendMessageL(TInt aMsgType, vncmsg& aMessage, char* aExtraData = NULL, TInt aDataLength = 0);

	// Receive an message
	IMPORT_C void RecvEventL(vncmsg& aEvent);

	// Receive a message
	IMPORT_C void RecvMessageL(vncmsg& aMessage);

	// Receive a message
	IMPORT_C void RecvExtraData(char* aExtraData, TInt aDataLength);
	
private:
	CVncIpc(TBool aServer);
	void ConstructL();
	
	ssize_t Send(int aFifo, void* aData, int aSize);
	ssize_t Recv(int aFifo, void* aData, int aSize);
	
private:
	TBool	iUI;			// Whether this is the UI server or not
	int		iFile;			// File to use for shared IPC objects
	key_t	iKeySem;		// The semaphore key
	int 	iSem;
	size_t	iMsgSize;		// The size of messages
	
	int		iEventFifo;		// The event fifo	
	int		iToUIFifo;		// The fifo for messages to the UI
	int		iFromUIFifo;	// The fifo for messages from the UI
	int		iQuitFifo;		// The quit fifo
	};



#endif

