/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

#include "VNCViewerApplication.h"
#include "VNCViewerAppUi.h"
#include "VNCViewerSettingsView.h"
#include "VNCViewerDocument.h"
#include "VNCViewer.hrh"
#include "VNCViewerExternalInterface.h"
#include "VNCViewAP.h"
#include <AknNoteWrappers.h>

#include <VNCViewer.rsg>
#include <BARSREAD.H>

/**
Creates and constructs the settings view.

@param aAppUi Reference to the AppUi
@return Pointer to a CVNCViewerSettingsView object
*/
CVNCViewerSettingsView* CVNCViewerSettingsView::NewLC(CVNCViewerDocument& aDocument)
	{
	CVNCViewerSettingsView* self = new (ELeave) CVNCViewerSettingsView(aDocument);
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
	}

/**
Constructor for the view.
Passes the application UI reference to the construction of the super class.

KNullViewId should normally be passed as parent view for the applications
default view. The parent view is the logical view that is normally activated
when a go back command is issued. KNullViewId will activate the system
default view.

@param aDocument Reference to the document
*/
CVNCViewerSettingsView::CVNCViewerSettingsView(CVNCViewerDocument& aDocument)
	: CAknView(), iDocument(aDocument)
	{
	}

/**
Destructor for the view
*/
CVNCViewerSettingsView::~CVNCViewerSettingsView()
	{
	if (iSettingsContainer)
		{
		AppUi()->RemoveFromViewStack(*this, iSettingsContainer);
		delete iSettingsContainer;
		iSettingsContainer = NULL;
		}
	}

/**
2nd stage construction of the view.
*/
void CVNCViewerSettingsView::ConstructL()
	{
	// Calls ConstructL that initialises the standard values.
	// This should always be called in the concrete view implementations.
	BaseConstructL(R_VNCVIEWER_SETTINGS_VIEW);
	}

/**
Returns the view Id

@return Returns the Uid of the view
*/
TUid CVNCViewerSettingsView::Id() const
	{
	return KUidVNCViewerSettingsView;
	}

/**
Activate the view

@param aPrevViewId The previous view id
@param aCustomMessageId A message Uid
@param aCustomMessage A custom message
*/
void CVNCViewerSettingsView::DoActivateL(const TVwsViewId& /* aPrevViewId */,
	                                  TUid /* aCustomMessageId */,
	                                  const TDesC8& /* aCustomMessage */)
	{
	if (!iSettingsContainer)
		{
		iSettingsContainer = CVNCViewerSettingsContainer::NewL(ClientRect(), iDocument);
		iSettingsContainer->SetMopParent(this);

		// Add the container to the control stack
		// So that it and its children can receive key events
		AppUi()->AddToViewStackL(*this, iSettingsContainer);

		// Complete the initialization
		iSettingsContainer->ActivateL();

		// Display the privacy
		TConnectionSettings& settings = iDocument.Settings();
		if (settings.iPrivacyWarning)
			{
			TPtrC privacyText(CEikonEnv::Static()->AllocReadResourceAsDes16LC(R_VNCVIEWER_PRIVACY)->Des());

			CAknNoteDialog* dialog = new (ELeave) CAknNoteDialog(CAknNoteDialog::EConfirmationTone, CAknNoteDialog::ENoTimeout);
			dialog->SetTextL(privacyText);
			if (dialog->ExecuteDlgLD(R_VNCVIEWER_PRIVACY_DIALOG))
				{
				settings.iPrivacyWarning = EFalse;
				iDocument.SaveIniFileL();
				}

			CleanupStack::PopAndDestroy(); // privacyText
			}
		}
	}

/**
 * View deactivation function intended for overriding by sub classes.
 * This function is called by  AknViewDeactivated().
 */
void CVNCViewerSettingsView::DoDeactivate()
	{
	if (iSettingsContainer)
		{
		AppUi()->RemoveFromViewStack(*this, iSettingsContainer);
		delete iSettingsContainer;
		iSettingsContainer = NULL;
		}
	}

/**
Handles all commands in the view.
Called by the UI framework when a command has been issued.
The command Ids are defined in the .hrh file.

@param aCommand The command to be executed
*/
void CVNCViewerSettingsView::HandleCommandL(TInt aCommand)
	{
	switch(aCommand)
		{
		case EVNCViewerConnectCmd:
			{
			TConnectionSettings& settings = iDocument.Settings();
			if (settings.iConnectionWarning)
				{
				TPtrC privacyText(CEikonEnv::Static()->AllocReadResourceAsDes16LC(R_VNCVIEWER_BILLABLE)->Des());

				CAknNoteDialog* dialog = new (ELeave) CAknNoteDialog(CAknNoteDialog::EConfirmationTone, CAknNoteDialog::ENoTimeout);
				dialog->SetTextL(privacyText);
				if (!dialog->ExecuteDlgLD(R_VNCVIEWER_BILLABLE_DIALOG))
					{
					break;
					}

				CleanupStack::PopAndDestroy(); // privacyText
				}

			SaveSettingsL();

			// Activates the vnc view
			dynamic_cast<CVNCViewerAppUi*>(AppUi())->SetActiveView(KUidVNCViewerVNCView);
			AppUi()->ActivateLocalViewL(KUidVNCViewerVNCView);
			break;
			}
		case EEikCmdExit:
		case EAknSoftkeyExit:
			AppUi()->Exit();
			break;
		default:
			AppUi()->HandleCommandL(aCommand);
			break;
		}
	}

/**
Save the settings to the TConnectionSettings structure in the document
*/
void CVNCViewerSettingsView::SaveSettingsL()
	{
	// Save the settings for the document
	iDocument.SaveIniFileL();
	}

/*
 * HandleStatusPaneSizeChange()
 * Event handler for status pane size changes.
 *
 */
void CVNCViewerSettingsView::HandleStatusPaneSizeChange()
	{
	if (iSettingsContainer)
		iSettingsContainer->SetRect(ClientRect());
	}


/**
The UI configuration has changed
*/
void CVNCViewerSettingsView::SizeChanged()
	{
	if (iSettingsContainer)
		{
		iSettingsContainer->SetRect(ClientRect());
		}
	}


//---------------------------------------------------------------------

/**
Creates and constructs the settings container.

@param aRect The bounding rectangle
@param aDocument The document
@return CVNCViewerSettingsContainer* The settings container
*/
CVNCViewerSettingsContainer* CVNCViewerSettingsContainer::NewL(const TRect& aRect, CVNCViewerDocument& aDocument)
	{
    CVNCViewerSettingsContainer* self = new (ELeave) CVNCViewerSettingsContainer();
    CleanupStack::PushL(self);
    self->ConstructL(aRect, aDocument);
    CleanupStack::Pop(self);
    return self;
	}

/**
2nd phase constructor

@param aRect The bounding rectangle
@param aDocument The document
*/
void CVNCViewerSettingsContainer::ConstructL(const TRect& aRect, CVNCViewerDocument& aDocument)
	{
    // Create a window for this application view
    CreateWindowL();

    // Set the windows size
    SetRect(aRect);

    // Create the Connect Setting List
    iSettingConnect = new (ELeave) CVNCViewerSettingsConnectList(aDocument);
    iSettingConnect->SetContainerWindowL(*this);
    iSettingConnect->ConstructL();

    // Create the Connect Setting List
    iSettingControl = new(ELeave) CVNCViewerSettingsControlList(aDocument);
    iSettingControl->SetContainerWindowL(*this);
    iSettingControl->ConstructL();

    // Create the Connect Setting List
    iSettingView = new(ELeave) CVNCViewerSettingsViewList(aDocument);
    iSettingView->SetContainerWindowL(*this);
    iSettingView->ConstructL();

    // Create the Connect Setting List
    iSettingEncodings = new(ELeave) CVNCViewerSettingsEncodingsList(aDocument);
    iSettingEncodings->SetContainerWindowL(*this);
    iSettingEncodings->ConstructL();

	//get the navigation
   	CEikStatusPane* sp = ((CAknAppUi*)iEikonEnv->EikAppUi())->StatusPane();
   	CAknNavigationControlContainer* np = (CAknNavigationControlContainer *)sp->ControlL(TUid::Uid(EEikStatusPaneUidNavi));

   	TResourceReader reader;
   	iCoeEnv->CreateResourceReaderLC(reader, R_VNCVIEWER_TAB_DISPLAY);
   	iNaviDecorator = np->CreateTabGroupL(reader);
   	CleanupStack::PopAndDestroy();  // resource reader

   	np->PushL(*iNaviDecorator); // Set new tab group.

    ActivateL();

	ChangeCurrentPage();

	DrawNow();
	}

/**
Constructor
*/
CVNCViewerSettingsContainer::CVNCViewerSettingsContainer()
	{
    // No implementation required
	}

/**
Destructor
*/
CVNCViewerSettingsContainer::~CVNCViewerSettingsContainer()
	{
	delete iSettingConnect;
	delete iSettingControl;
	delete iSettingView;
	delete iSettingEncodings;

	CEikStatusPane* sp = ((CAknAppUi*)iEikonEnv->EikAppUi())->StatusPane();
	CAknNavigationControlContainer* np= (CAknNavigationControlContainer *)sp->ControlL(TUid::Uid(EEikStatusPaneUidNavi));
	np->Pop();
	delete iNaviDecorator;
	}


/**
Draw the container

@param aRect the rectangle to draw within
*/
void CVNCViewerSettingsContainer::Draw(const TRect& /*aRect*/) const
	{
    // Get the standard graphics context
    CWindowGc& gc = SystemGc();

    // Gets the control's extent
    TRect drawRect(Rect());

    // Clears the screen
    gc.Clear(drawRect);
	}

/**
The size of the container has changed
*/
void CVNCViewerSettingsContainer::SizeChanged()
	{
    DrawNow();
	}

/**
The number of component controls

@return TInt The number of component controls
*/
TInt CVNCViewerSettingsContainer::CountComponentControls() const
	{
	return iSettingConnect ? 1 : 0;
	}

/**
Return a component control

@return CCoeControl The component control
*/
CCoeControl* CVNCViewerSettingsContainer::ComponentControl(TInt /* aIndex */) const
	{
	switch(iCurrentPage)
		{
		case 0:
			return iSettingConnect;
		case 1:
			return iSettingControl;
		case 2:
			return iSettingView;
		case 3:
			return iSettingEncodings;
		default:
			return NULL;
		}
	}

/**
A key event has occurred, move the page

@param aKeyEvent The key event
@param aType The type of key event
@return TKeyResponse Whether the keyboard event has been consumed
*/
TKeyResponse CVNCViewerSettingsContainer::OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType)
	{
	if (CountComponentControls() == 0)
		return EKeyWasNotConsumed;

	if (aType == EEventKey)
		{
   		switch (aKeyEvent.iScanCode)
	   		{
      		case EStdKeyRightArrow:
         		++iCurrentPage;
         		ChangeCurrentPage();
	         	break;
      		case EStdKeyLeftArrow:
         		--iCurrentPage;
         		ChangeCurrentPage();
	         	break;
	   		}
		}

   	switch(iCurrentPage)
	   	{
      	case 0:   return iSettingConnect->OfferKeyEventL(aKeyEvent, aType);
      	case 1:   return iSettingControl->OfferKeyEventL(aKeyEvent, aType);
      	case 2:   return iSettingView->OfferKeyEventL(aKeyEvent, aType);
      	case 3:   return iSettingEncodings->OfferKeyEventL(aKeyEvent, aType);
	   	}
   	return EKeyWasNotConsumed;
	}

/**
Change the current page
*/
void CVNCViewerSettingsContainer::ChangeCurrentPage()
	{
   	iCurrentPage = Max(0, iCurrentPage);
   	iCurrentPage = Min(3, iCurrentPage);

	// Hide all setting listboxes
   	iSettingConnect->MakeVisible(EFalse);
   	iSettingControl->MakeVisible(EFalse);
   	iSettingView->MakeVisible(EFalse);
   	iSettingEncodings->MakeVisible(EFalse);

	// Change the tab
   	CAknTabGroup* tabGroup = static_cast<CAknTabGroup*>(iNaviDecorator->DecoratedControl());

   	switch(iCurrentPage)
 	  	{
		case 0:
         	iSettingConnect->MakeVisible(ETrue);
         	iSettingConnect->ActivateL();
         	tabGroup->SetActiveTabById(EVncViewerConnectPage);
    	    break;
      	case 1:
        	iSettingControl->MakeVisible(ETrue);
         	iSettingControl->ActivateL();
         	tabGroup->SetActiveTabById(EVncViewerControlPage);
       		break;
      	case 2:
         	iSettingView->MakeVisible(ETrue);
         	iSettingView->ActivateL();
         	tabGroup->SetActiveTabById(EVncViewerViewPage);
	        break;
      	case 3:
         	iSettingEncodings->MakeVisible(ETrue);
         	iSettingEncodings->ActivateL();
         	tabGroup->SetActiveTabById(EVncViewerEncodingsPage);
	        break;
	   	}

   	DrawNow();
	}

//------------------------------------------------------

/**
Creates the connect settings list

@param aDocument The document
*/
CVNCViewerSettingsConnectList::CVNCViewerSettingsConnectList(CVNCViewerDocument& aDocument)
: CAknSettingItemList(), iDocument(aDocument), iVia(EVncSettingViaPreferred)
	{
	}

/**
2nd phase constructor
*/
void CVNCViewerSettingsConnectList::ConstructL()
	{
    ConstructFromResourceL(R_VNCVIEWER_CONNECT_SETTING_LIST);

	TConnectionSettings& settings = iDocument.Settings();

   	// Create an array of access points
	iAccessPoints = new(ELeave) CDesCArrayFlat(4);
	CVNCViewAccessPoint::PopulateAccessPointsL(*iAccessPoints);

	TPtrC existingPreferred(CEikonEnv::Static()->AllocReadResourceAsDes16LC(R_PREFERRED_ACCESS_POINT)->Des());

	// Create a new array
	CArrayPtr<CAknEnumeratedText>* enumArray = new (ELeave) CArrayPtrFlat<CAknEnumeratedText>(iAccessPoints->MdcaCount() + 1);
	CleanupStack::PushL(enumArray);
	CArrayPtr<HBufC>* popupArray = new (ELeave) CArrayPtrFlat<HBufC>(iAccessPoints->MdcaCount() + 1);
	CleanupStack::PushL(popupArray);

	// Now add the preferred
	iVia = EVncSettingViaPreferred;
	TInt id = iVia;

	HBufC* enumText = existingPreferred.AllocLC();
	CAknEnumeratedText* itemText = new (ELeave) CAknEnumeratedText(id, enumText);
	CleanupStack::Pop(enumText);
	CleanupStack::PushL(itemText);
	enumArray->AppendL(itemText);
	CleanupStack::Pop(itemText); // itemText

	HBufC* popupText = existingPreferred.AllocLC();
	popupArray->AppendL(popupText);
	CleanupStack::Pop(popupText);

	for (TInt count = 0; count < iAccessPoints->MdcaCount(); ++count)
		{
		HBufC* enumText = iAccessPoints->MdcaPoint(count).AllocLC();
		CAknEnumeratedText* itemText = new (ELeave) CAknEnumeratedText(++id, enumText);
		CleanupStack::Pop(enumText);
		CleanupStack::PushL(itemText);
		enumArray->AppendL(itemText);
		CleanupStack::Pop(itemText);

		HBufC* popupText = iAccessPoints->MdcaPoint(count).AllocLC();
		popupArray->AppendL(popupText);
		CleanupStack::Pop(popupText);

		if (settings.iVia.CompareF(iAccessPoints->MdcaPoint(count)) == 0)
			iVia = id;
		}

	// Now set the new data
	iViaControl->SetEnumeratedTextArrays(enumArray, popupArray);
	CleanupStack::Pop(2); // popupArray, enumArray
	CleanupStack::PopAndDestroy(1); // existingPreferred
	iViaControl->HandleTextArrayUpdateL();
    iViaControl->UpdateListBoxTextL();
	}

/**
Destructor
*/
CVNCViewerSettingsConnectList::~CVNCViewerSettingsConnectList()
	{
	delete iAccessPoints;
	}

/**
Create the items in the setting list

@param aIdentifier The setting item identifier
@return CAknSettingItem* The setting control
*/
CAknSettingItem* CVNCViewerSettingsConnectList::CreateSettingItemL(TInt aIdentifier)
	{
	CAknSettingItem* settingItem = NULL;

	TConnectionSettings& settings = iDocument.Settings();

	switch (aIdentifier)
		{
		case EConnectViaCtrl:
			{
			iViaControl = new (ELeave) CAknEnumeratedTextPopupSettingItem(aIdentifier, iVia);
			settingItem = iViaControl;
			}
			break;
		case EConnectAddressCtrl:
			settingItem = new (ELeave) CAknTextSettingItem(aIdentifier, settings.iAddress);
			break;
		case EConnectPortCtrl:
			settingItem = new (ELeave) CAknIntegerEdwinSettingItem(aIdentifier, settings.iPort);
			break;
		case EConnectPrivacyCtrl:
			settingItem = new (ELeave) CAknBinaryPopupSettingItem(aIdentifier, settings.iPrivacyWarning);
			break;
		case EConnectBillableCtrl:
			settingItem = new (ELeave) CAknBinaryPopupSettingItem(aIdentifier, settings.iConnectionWarning);
			break;
		}
	return settingItem;
	}

/**
Edit an item in the list

@param aIndex The index of the item to edit
@param aCalledFromMenu Whether the item is called from a menu
*/
void CVNCViewerSettingsConnectList::EditItemL(TInt aIndex, TBool aCalledFromMenu)
	{
	CAknSettingItemList::EditItemL(aIndex, aCalledFromMenu);
	(*SettingItemArray())[aIndex]->StoreL();

	if (iAccessPoints)
		{
		TConnectionSettings& settings = iDocument.Settings();
		if (iVia == EVncSettingViaPreferred)
			{
			settings.iVia.SetLength(0);
			}
		else
			{
			settings.iVia.Copy(iAccessPoints->MdcaPoint(iVia - 1));
			}
		}
	}

//------------------------------------------------------

/**
Creates the control settings list

@param aDocument The document
*/
CVNCViewerSettingsControlList::CVNCViewerSettingsControlList(CVNCViewerDocument& aDocument)
: CAknSettingItemList(), iDocument(aDocument)
	{
	}

/**
2nd phase constructor
*/
void CVNCViewerSettingsControlList::ConstructL()
	{
    ConstructFromResourceL(R_VNCVIEWER_CONTROL_SETTING_LIST);
	UpdateState();
	}

/**
Create the items in the setting list

@param aIdentifier The setting item identifier
@return CAknSettingItem* The setting control
*/
CAknSettingItem* CVNCViewerSettingsControlList::CreateSettingItemL(TInt aIdentifier)
	{
	CAknSettingItem* settingItem = NULL;

	TConnectionSettings& settings = iDocument.Settings();

	switch (aIdentifier)
		{
		case EControlViewOnlyCtrl:
			settingItem = new (ELeave) CAknBinaryPopupSettingItem(aIdentifier, settings.iViewOnly);
			break;
		case EControlSharedCtrl:
			settingItem = new (ELeave) CAknBinaryPopupSettingItem(aIdentifier, settings.iShared);
			break;
		case EControlCursorCtrl:
			settingItem = new (ELeave) CAknBinaryPopupSettingItem(aIdentifier, settings.iCursor);
			break;
		case EControlX11CursorCtrl:
			settingItem = new (ELeave) CAknBinaryPopupSettingItem(aIdentifier, settings.iX11Cursor);
			iX11Cursor = settingItem;
			break;
		}
	return settingItem;
	}

/**
Edit an item in the list

@param aIndex The index of the item to edit
@param aCalledFromMenu Whether the item is called from a menu
*/
void CVNCViewerSettingsControlList::EditItemL(TInt aIndex, TBool aCalledFromMenu)
	{
	CAknSettingItemList::EditItemL(aIndex, aCalledFromMenu);
	(*SettingItemArray())[aIndex]->StoreL();

    UpdateState();
    HandleChangeInItemArrayOrVisibilityL();
	}

/**
Update the state of the settings
*/
void CVNCViewerSettingsControlList::UpdateState()
	{
    // The items have been created, now alter the state of X11 depending upon the state of the cursor
	TConnectionSettings& settings = iDocument.Settings();
    iX11Cursor->SetHidden(!settings.iCursor);
	}


//------------------------------------------------------

/**
Creates the view settings list

@param aDocument The document
*/
CVNCViewerSettingsViewList::CVNCViewerSettingsViewList(CVNCViewerDocument& aDocument)
: CAknSettingItemList(), iDocument(aDocument)
	{
	}

/**
2nd phase constructore
*/
void CVNCViewerSettingsViewList::ConstructL()
	{
    ConstructFromResourceL(R_VNCVIEWER_VIEW_SETTING_LIST);
	}

/**
Create the items in the setting list

@param aIdentifier The setting item identifier
@return CAknSettingItem* The setting control
*/
CAknSettingItem* CVNCViewerSettingsViewList::CreateSettingItemL(TInt aIdentifier)
	{
	CAknSettingItem* settingItem = NULL;

	TConnectionSettings& settings = iDocument.Settings();
	switch (aIdentifier)
		{
		case EViewFullScreenCtrl:
			settingItem = new (ELeave) CAknBinaryPopupSettingItem(aIdentifier, settings.iFullScreen);
			break;
		case EViewDepthCtrl:
			settingItem = new (ELeave) CAknEnumeratedTextPopupSettingItem(aIdentifier, settings.iDepth);
			break;
		}
	return settingItem;
	}

/**
Edit an item in the list

@param aIndex The index of the item to edit
@param aCalledFromMenu Whether the item is called from a menu
*/
void CVNCViewerSettingsViewList::EditItemL(TInt aIndex, TBool aCalledFromMenu)
	{
	CAknSettingItemList::EditItemL(aIndex, aCalledFromMenu);
	(*SettingItemArray())[aIndex]->StoreL();
	}

//------------------------------------------------------

/**
Creates the encoding settings list

@param aDocument The document
*/
CVNCViewerSettingsEncodingsList::CVNCViewerSettingsEncodingsList(CVNCViewerDocument& aDocument)
: CAknSettingItemList(), iDocument(aDocument)
	{
	}

/**
2nd phase constructore
*/
void CVNCViewerSettingsEncodingsList::ConstructL()
	{
    ConstructFromResourceL(R_VNCVIEWER_ENCODINGS_SETTING_LIST);
    UpdateState();
	}

/**
Create the items in the setting list

@param aIdentifier The setting item identifier
@return CAknSettingItem* The setting control
*/
CAknSettingItem* CVNCViewerSettingsEncodingsList::CreateSettingItemL(TInt aIdentifier)
	{
	CAknSettingItem* settingItem = NULL;

	TConnectionSettings& settings = iDocument.Settings();
	switch (aIdentifier)
		{
		case EViewEncodingsCtrl:
			settingItem = new (ELeave) CAknEnumeratedTextPopupSettingItem(aIdentifier, settings.iEncodings);
			break;
		case EViewJpegCtrl:
			settingItem = new (ELeave) CAknBinaryPopupSettingItem(aIdentifier, settings.iJpeg);
			iJpeg = settingItem;
			break;
		case EViewCompressionCtrl:
			settingItem = new (ELeave) CAknIntegerEdwinSettingItem(aIdentifier, settings.iCompression);
			iCompression = settingItem;
			break;
		case EViewQualityCtrl:
			settingItem = new (ELeave) CAknIntegerEdwinSettingItem(aIdentifier, settings.iQuality);
			iQuality = settingItem;
			break;
		}

	return settingItem;
	}

/**
Edit an item in the list

@param aIndex The index of the item to edit
@param aCalledFromMenu Whether the item is called from a menu
*/
void CVNCViewerSettingsEncodingsList::EditItemL(TInt aIndex, TBool aCalledFromMenu)
	{
	CAknSettingItemList::EditItemL(aIndex, aCalledFromMenu);
	(*SettingItemArray())[aIndex]->StoreL();

	UpdateState();
    HandleChangeInItemArrayOrVisibilityL();
	}

/**
Update the state of the settings
*/
void CVNCViewerSettingsEncodingsList::UpdateState()
	{
    // The items have been created, now alter the state of X11 depending upon the state of the cursor
	TConnectionSettings& settings = iDocument.Settings();

	if ((settings.iEncodings == EVncSettingEncodingAny) || (settings.iEncodings == EVncSettingEncodingTight))
		{
		iJpeg->SetHidden(EFalse);
		if (settings.iJpeg)
			{
			iCompression->SetHidden(EFalse);
			iQuality->SetHidden(EFalse);
			}
		else
			{
			iCompression->SetHidden(ETrue);
			iQuality->SetHidden(ETrue);
			}
		}
	else
		{
		iJpeg->SetHidden(ETrue);
		iCompression->SetHidden(ETrue);
		iQuality->SetHidden(ETrue);
		}
	}

