/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */


#include <QikCommand.h>
#include <QikFloatingPointEditor.h>

#include "VNCScaleDialog.h"
#include "VNCViewer.hrh"
#include <VNCViewer.rsg>

/**
Creates, constructs, execute and destroys the dialog.

@param aScale The scale
*/
CVNCScaleDialog* CVNCScaleDialog::NewL(TReal aScale)
	{
	CVNCScaleDialog* self = new (ELeave) CVNCScaleDialog(aScale);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
	}

/**
Constructor for the dialog.

@param aScale The scale
*/
CVNCScaleDialog::CVNCScaleDialog(TReal aScale) 
	: iCmdManager(CQikCommandManager::Static(*iCoeEnv)), iInitialScale(aScale)
	{
	}

/**
Destructor for the dialog
*/
CVNCScaleDialog::~CVNCScaleDialog()
	{
	}

/**
2nd stage construction of the dialog.
*/
void CVNCScaleDialog::ConstructL()
	{
	PrepareL(R_VNCVIEWER_SCALE_DIALOG);
	}

/**
Initialize the controls
*/
void CVNCScaleDialog::PreLayoutDynInitL()
	{
	CQikFloatingPointEditor* scaleControl = LocateControlByUniqueHandle<CQikFloatingPointEditor>(EVNCViewerScaleDialogEditor);
	scaleControl->SetValueL(iInitialScale);
	}

/**
Get the capabilities of the dialog

@return TCoeInputCapabilities The capabilites
*/
TCoeInputCapabilities CVNCScaleDialog::InputCapabilities() const
	{
	return TCoeInputCapabilities::EWesternNumericReal;
	}

/**
Handles all commands in the dialog.
Called by the UI framework when a command has been issued.
The command Ids are defined in the .hrh file.

@param aCommand The command to be executed
*/
void CVNCScaleDialog::HandleCommandL(CQikCommand& aCommand)
	{
	switch(aCommand.Id())
		{
		case EVNCViewerScaleCmdDone:
		case EVNCViewerScaleCmdCancel:
			{
			// Signals that the dialog should close with the command id as value
			CloseDialog(aCommand.Id());
			break;
			}
		default:
			{
			CQikSimpleDialog::HandleCommandL(aCommand);
			break;
			}
		}
	}

/**
Return the text from the editor

@return The new scale
*/
const TReal CVNCScaleDialog::ValueL()
	{
	const CQikFloatingPointEditor* scaleControl = LocateControlByUniqueHandle<const CQikFloatingPointEditor>(EVNCViewerScaleDialogEditor);
	return scaleControl->Value();
	}

/**
Cancel the dialog
*/
void CVNCScaleDialog::Cancel()
	{
	CloseDialog(KErrCancel);
	}

//---------------------------------------------------------------
//---------------------------------------------------------------

/**
Constructor for the active object.

@param aScaleCallback The callback object
@param aScale The initial scale
*/
CScaleActive* CScaleActive::NewL(MScale &aScaleCallback, TReal aScale)
	{
	CScaleActive* self = new(ELeave) CScaleActive(aScaleCallback);
	CleanupStack::PushL(self);
	self->ConstructL(aScale);
	CleanupStack::Pop(self);
	return self;
	}

/**
Destrutor for the active object.
*/
CScaleActive::~CScaleActive()
	{
	Cancel();
	delete iDialog;
	}

/**
Constructor for the active object.

@param aScaleCallback The callback object
*/
CScaleActive::CScaleActive(MScale &aScaleCallback)
: CActive(EPriorityStandard), iScaleCallback(aScaleCallback)
	{
	CActiveScheduler::Add(this);
	}

/**
2nd phase constructor
*/
void CScaleActive::ConstructL(TReal aScale)
	{
	// Create the dialog
	iDialog = CVNCScaleDialog::NewL(aScale * 100);
	iStatus = KRequestPending;
	iDialog->RunL(iStatus);
	SetActive();	
	}
	
/**
The dialog has completed
*/
void CScaleActive::RunL()
	{
	iScaleCallback.ScaleL(iStatus.Int(), iDialog->ValueL() / 100);
	}
	
/**
The dialog needs to be cancelled
*/
void CScaleActive::DoCancel()
	{
	iDialog->Cancel();	
	iScaleCallback.ScaleL(KErrCancel, 0);
	}
	
