// VNCViewerVNCView.cpp
/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

#include <QikCommand.h>
#include <DeviceKeys.h>
#include "VNCViewerApplication.h"
#include "VNCViewerAppUi.h"
#include "VNCViewerVNCView.h"
#include "VNCViewerSettingsView.h"
#include "VNCViewerDocument.h"
#include "VNCViewer.hrh"
#include "VNCViewerExternalInterface.h"
#include "VNCPasswordDialog.h"
#include <utf.h>
#include <VNCViewer.rsg>

/**
Scroll defaults
*/
// The number of pixels at the edge of the view to trigger a bump 
const TInt scrollBumpMargin = 10;

// The number of pixels to bump
const TInt scrollBump = 20;

// The number of pixels to move the cursor
const TInt moveCursor = 5;

// Scales
const TReal absoluteScale = 0.0;
const TReal fullscreenScale = 1.0;

/**
Creates and constructs the view.

@param aAppUi Reference to the AppUi
@return CVNCViewerVNCView* Pointer to a CVNCViewerVNCView object
*/
CVNCViewerVNCView* CVNCViewerVNCView::NewLC(CQikAppUi& aAppUi)
	{
	CVNCViewerVNCView* self = new (ELeave) CVNCViewerVNCView(aAppUi);
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
	}

/**
Constructor for the view.
Passes the application UI reference to the construction of the super class.

KNullViewId should normally be passed as parent view for the applications 
default view. The parent view is the logical view that is normally activated 
when a go back command is issued. KNullViewId will activate the system 
default view. 

@param aAppUi Reference to the application UI
*/
CVNCViewerVNCView::CVNCViewerVNCView(CQikAppUi& aAppUi) 
	: CQikViewBase(aAppUi, KNullViewId), iConfigClient(CQUiConfigClient::Static()), iNeedToConnect(ETrue)
	{
	}

/**
Destructor for the view
*/
CVNCViewerVNCView::~CVNCViewerVNCView()
	{
	}

/**
2nd stage construction of the view.
*/
void CVNCViewerVNCView::ConstructL()
	{
	// Calls ConstructL that initialises the standard values. 
	// This should always be called in the concrete view implementations.
	BaseConstructL();
	}
	
/**
Inherited from CQikViewBase and called upon by the UI Framework. 
It creates the view with commands from resource.
*/
void CVNCViewerVNCView::ViewConstructL()
	{
	// Loads information about the UI configurations this view supports 
	// together with definition of each view, its layout and commands.
	ViewConstructFromResourceL(R_VNCVIEWER_UI_VNC_CONFIGURATIONS);
	}

/**
Returns the view Id

@return Returns the Uid of the view
*/
TVwsViewId CVNCViewerVNCView::ViewId()const
	{
	return TVwsViewId(KUidVNCViewerApp, KUidVNCViewerVNCView);
	}

/**
Handles all commands in the view.
Called by the UI framework when a command has been issued.
The command Ids are defined in the .hrh file.

@param aCommand The command to be executed
@see CQikViewBase::HandleCommandL
*/
void CVNCViewerVNCView::HandleCommandL(CQikCommand& aCommand)
	{
	if ((iExiting) || (!iVNCContainer))
		return;
	
	switch(aCommand.Id())
		{
		case EVNCViewerScaleFullScreen:
			iVNCContainer->SetScaleL(fullscreenScale);
			break;
		case EVNCViewerScale3_4:
			iVNCContainer->SetScaleL(0.75);
			break;
		case EVNCViewerScale1_2:
			iVNCContainer->SetScaleL(0.5);
			break;
		case EVNCViewerScale1_4:
			iVNCContainer->SetScaleL(0.25);
			break;
		case EVNCViewerScaleAccurate:
			iVNCContainer->SetScaleL(absoluteScale);
			break;
		case EVNCViewerScaleCustom:
			iVNCContainer->SetCustomScaleL();
			break;
		case EVNCViewerSendText:
			iVNCContainer->SendTextDialogL();
			break;
		case EVNCViewerFindCursor:
			iVNCContainer->FindCursorL();
			break;
		case EVNCViewerDirectionKeysCursor:
			iVNCContainer->SetArrowKeysControlCursor(ETrue);
			break;
		case EVNCViewerDirectionKeysCaret:
			iVNCContainer->SetArrowKeysControlCursor(EFalse);
			break;
		case EVNCViewerModifierShift:
		case EVNCViewerModifierCtrl:
		case EVNCViewerModifierAlt:
		case EVNCViewerModifierFunc:
			break;
		case EVNCViewerMouseLeft:
			iVNCContainer->SetButtonMode(CVNCViewerScrollableContainer::EButtonLeft);
			break;			
		case EVNCViewerMouseMiddle:
			iVNCContainer->SetButtonMode(CVNCViewerScrollableContainer::EButtonMiddle);
			break;			
		case EVNCViewerMouseRight:
			iVNCContainer->SetButtonMode(CVNCViewerScrollableContainer::EButtonRight);
			break;	
		case EVNCViewerOrientationLandscape:
			{	
			iConfigClient.SetScreenModeL(EQikUiConfigLandscape);
			}
			break;
		case EVNCViewerOrientationPortrait:
			{
			iConfigClient.SetScreenModeL(EQikUiConfigPortrait);
			}
			break;
		case EVNCViewerRefresh:
			iVNCContainer->RefreshL();
			break;				
		case EVNCViewerAbout:
			CEikonEnv::Static()->InfoWinL(R_ABOUT_TITLE, R_ABOUT_TEXT);
			break;
		case EEikCmdExit:
		case EQikCmdGoBack:
			iExiting = ETrue;
			if (iVNCContainer->PostQuit())
				CActiveScheduler::Start();
			
			if (aCommand.Id() == EQikCmdGoBack)
				dynamic_cast<CVNCViewerAppUi&>(iQikAppUi).SetActiveView(KUidVNCViewerSettingsView);
			CQikViewBase::HandleCommandL(aCommand);
			break;
		// Go back and exit command will be passed to the CQikViewBase to handle.
		default:
			// Check if we are sending a key
			if (!iVNCContainer->SendKeyL(aCommand.Id()))
				CQikViewBase::HandleCommandL(aCommand);
			break;
		}
	}

void CVNCViewerVNCView::Shutdown()
	{
	if (iVNCContainer->PostQuit())	
		CActiveScheduler::Start();
	}

/**
When the application needs to display a view, the view framework calls 
the view's ViewActivatedL() function. In some cases, the view is also passed 
a parameter that tells it what to display. In this view page1 is activated 
when the view gets visible.

@param aPrevViewId The composite view id of the previous view
@param aCustomMessageId The id of the message
@param aCustomMessage The message
*/
void CVNCViewerVNCView::ViewActivatedL(const TVwsViewId& aPrevViewId, const TUid aCustomMessageId, const TDesC8& aCustomMessage)
	{
	CQikViewBase::ViewActivatedL(aPrevViewId, aCustomMessageId, aCustomMessage);
	
    TQikViewMode viewMode;
	viewMode.SetAppTitleBar(ETrue);
	viewMode.SetButtonOrSoftkeyBar(ETrue);
	viewMode.SetStatusBar(ETrue);
	viewMode.SetToolbar(ETrue);
    viewMode.SetNormal();
    SetViewModeL(viewMode); 

	// Connect for the first time only
	if (iNeedToConnect)
		{
		CVNCViewerDocument* document = static_cast<CVNCViewerDocument*>(iQikAppUi.Document());

		// Create the container
		iVNCContainer = new (ELeave) CVNCViewerScrollableContainer(*this, *document);
		Controls().AppendLC(iVNCContainer, EVNCViewerContainer);
		iVNCContainer->ConstructL(Rect());
		CleanupStack::Pop(iVNCContainer); 	

		CQikCommandManager& manager = CQikCommandManager::Static();
		if (document->Settings().iFullScreen)
			{
			manager.SetChecked(*this, EVNCViewerScaleFullScreen, ETrue);			
			}
		else
			{
			manager.SetChecked(*this, EVNCViewerScaleAccurate, ETrue);			
			}
		
		iVNCContainer->SetRect(Rect());
		iVNCContainer->ConnectL();
		iNeedToConnect = EFalse;
		}
	}

/**
Forward on the foreground event

@param aForeground Foregrounnd or Background?
*/
void CVNCViewerVNCView::HandleForegroundEventL(TBool aForeground)
	{
	if ((iVNCContainer) && (iVNCContainer->LoggedOn()))
		iVNCContainer->HandleForegroundEventL(aForeground);
	}

/**
The UI configuration has changed
*/
void CVNCViewerVNCView::SizeChanged()
	{
	if (iVNCContainer)
		iVNCContainer->SetSize(Size());
	}

/**
Force a quit
*/
void CVNCViewerVNCView::QuitNow()
	{
	CActiveScheduler::Stop();
	DestroyEngine(EFalse);
	}

/**
The engine should be destroyed

@param aSwitchView Whether to switch to the settings view
*/
void CVNCViewerVNCView::DestroyEngine(TBool aSwitchView)
	{
	if (aSwitchView)
		{
		dynamic_cast<CVNCViewerAppUi&>(iQikAppUi).SetActiveView(KUidVNCViewerSettingsView);
		TVwsViewId viewId = TVwsViewId(KUidVNCViewerApp, KUidVNCViewerSettingsView);
		TRAPD(err, iQikAppUi.ActivateViewL(viewId));
		}

	RemoveControl(*iVNCContainer);
	delete iVNCContainer;
	iVNCContainer = NULL;	
	iNeedToConnect = ETrue;
	}

/**
Handle a change to the configuration
*/
void CVNCViewerVNCView::HandleUiConfigChangedL()	
	{	
	CQikViewBase::HandleUiConfigChangedL();
	
	TQikUiConfig config = iConfigClient.CurrentConfig(); 

	CQikCommandManager& manager = CQikCommandManager::Static();
	if (config.InteractionStyle() == EQikInteractionStyleMenubar)
		{
		manager.SetAvailable(*this, EVNCViewerDirectionKeys, EFalse);
		manager.SetAvailable(*this, EVNCViewerOrientation, ETrue);
		}
	else
		{
		manager.SetAvailable(*this, EVNCViewerDirectionKeys, ETrue);		
		manager.SetAvailable(*this, EVNCViewerOrientation, EFalse);
		manager.SetChecked(*this, EVNCViewerDirectionKeysCursor, ETrue);
		iVNCContainer->SetArrowKeysControlCursor(ETrue);
		}
	}

//---------------------------------------------------------------
//---------------------------------------------------------------



/**
Creates and constructs the viewer container (including a scroll bar).

@param aAppUi Reference to the AppUi
@param aDocument The document
*/
CVNCViewerScrollableContainer::CVNCViewerScrollableContainer(CVNCViewerVNCView& aView, CVNCViewerDocument& aDocument)
	: iView(aView), iDocument(aDocument), iArrowKeypressMovesPointer(ETrue), iPointerAcceleration(1), iCursorLocated(EFalse)
	{
	}

/**
2nd stage construction of the container.

@param aRect The bounding rectangle
*/
void CVNCViewerScrollableContainer::ConstructL(const TRect& /* aRect */)
	{
	CQikScrollableContainer::ConstructL(ETrue);
	SetUniqueHandle(EVNCViewerContainer);

	// Create the viewport
	iVNCViewPortWindow = CVNCViewportWindow::NewL(*this);

	BeginUpdateLC();	
	AddControlLC(iVNCViewPortWindow, EVNCViewerViewport);
	CleanupStack::Pop(iVNCViewPortWindow); 
	EndUpdateL();

	// Set the scroll bar visibility
	ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto,
                                                CEikScrollBarFrame::EAuto);

	ActivateL();
	DrawNow();
	}

/**
Destructor for the container
*/
CVNCViewerScrollableContainer::~CVNCViewerScrollableContainer()
	{
	BeginUpdateLC();	
	RemoveControl(*iVNCViewPortWindow);
	EndUpdateL();
	
	ResetAndDestroy();
	Destroy();
	delete iVNCViewPortWindow;
	delete iSendTextActive;
	delete iScaleActive;
	}

/**
Destroy the engine
*/
void CVNCViewerScrollableContainer::Destroy()
	{
	iCursor.Close();
	delete iCursorBitmap;
	iCursorBitmap = NULL;
	delete iCursorMask;
	iCursorMask = NULL;
	delete iVNCViewMsgProc;
	iVNCViewMsgProc = NULL;
	delete iMainBitmap;
	iMainBitmap = NULL;
	iLoggedOn = EFalse;
	}

/**
From MVNCViewerEvent, Show a password dialog

@return HBufC8* The password
*/
HBufC8* CVNCViewerScrollableContainer::ShowPasswordL()
	{
	TBuf<16> password;
	CVNCPasswordDialog::RunDlgLD(password);

	// Set the commands as available
	CQikCommandManager& manager = CQikCommandManager::Static();
	manager.SetAvailable(*this, EVNCViewerSendText, ETrue);
	manager.SetAvailable(*this, EVNCViewerClick, ETrue);
	manager.SetAvailable(*this, EVNCViewerSendKey, ETrue);
	manager.SetAvailable(*this, EVNCViewerModifiers, ETrue);
	manager.SetAvailable(*this, EVNCViewerMouse, ETrue);
	manager.SetAvailable(*this, EVNCViewerScale, ETrue);
	manager.SetAvailable(*this, EVNCViewerCtrlAltDel, ETrue);
	manager.SetAvailable(*this, EVNCViewerRefresh, ETrue);

	CQUiConfigClient& configClient = CQUiConfigClient::Static();
	TQikUiConfig config = configClient.CurrentConfig(); 

	if (config.InteractionStyle() == EQikInteractionStyleMenubar)
		{
		manager.SetAvailable(*this, EVNCViewerDirectionKeys, EFalse);
		manager.SetAvailable(*this, EVNCViewerOrientation, ETrue);
		}
	else
		{
		manager.SetAvailable(*this, EVNCViewerDirectionKeys, ETrue);
		manager.SetAvailable(*this, EVNCViewerOrientation, ETrue);
		}

	if (config.ScreenMode() == EQikScreenModePortrait)
		manager.SetChecked(*this, EVNCViewerOrientationPortrait, ETrue);
	else
		manager.SetChecked(*this, EVNCViewerOrientationLandscape, ETrue);
	
	if (iArrowKeypressMovesPointer)
		manager.SetChecked(*this, EVNCViewerDirectionKeysCursor, ETrue);
	else
		manager.SetChecked(*this, EVNCViewerDirectionKeysCaret, ETrue);
	
	manager.SetChecked(*this, EVNCViewerMouseLeft, ETrue);
	manager.SetAvailable(*this, EVNCViewerFindCursor, iDocument.Settings().iCursor);			

	return CnvUtfConverter::ConvertFromUnicodeToUtf8L(password);
	}

/**
From MVNCViewerEvent, Create the bitmap

@param aHandle The bitmap handle
*/
void CVNCViewerScrollableContainer::CreateBitmapL(TInt aHandle)
	{
	// Create the bitmap and activate it	
	iMainBitmap = new(ELeave) CFbsBitmap();
	User::LeaveIfError(iMainBitmap->Duplicate(aHandle));
	
	TSize bitmapSize = iMainBitmap->SizeInPixels();
		
	// Now set the viewport size
	if (iScale == fullscreenScale)
		{
		iVNCViewPortWindow->SetRect(Rect());	

		// Set the scroll bar visibility
		ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff,
                                                CEikScrollBarFrame::EOff);
		}
	else
		{
		TRect viewPortSize(0, 0, bitmapSize.iWidth, bitmapSize.iHeight);
		iVNCViewPortWindow->SetRect(viewPortSize);
		
		// Set the scroll bar visibility
		ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto,
                                                CEikScrollBarFrame::EAuto);
		}
		
	// Change the size of the page size
	SetPageSize(MinimumPageSize());
	UpdateScrollBarsL();

	// Send the graphics expose event
	iVNCViewMsgProc->SendGraphicsExposeEventL(0, 0, bitmapSize.iWidth, bitmapSize.iHeight);
	
	// We're now logged on
	iLoggedOn = ETrue;	
	}

/**
From MVNCViewerEvent, Copy the bitmap onto the screen

@param aRect The bounding rectangle
*/
void CVNCViewerScrollableContainer::PutImage(const TRect& aRect)
	{		
	DrawNow();
	if (iScale == fullscreenScale)
		iVNCViewPortWindow->DrawNow();
	else
		{
		TRect scaledRect = ScaledRect(aRect);
		iVNCViewPortWindow->DrawNow(scaledRect);
		}
	}
	
/**
From MVNCViewerEvent, The engine has ended
*/
void CVNCViewerScrollableContainer::EngineQuit()
	{
	iView.QuitNow();
	}

/**
From MVNCViewerEvent, The engine is moving the soft cursor

@param aX The cursor X position
@param aY The cursor Y position
*/
void CVNCViewerScrollableContainer::MoveSoftCursorL(TInt aX, TInt aY)
	{
	iCursorPos.iX = aX;
	iCursorPos.iY = aY;
	if (!iCursorLocated)
		{
		FindCursorL();
		iCursorLocated = ETrue;
		}

	DrawNow();
	}

/**
From MVNCViewerEvent, The engine is asking the UI to create a cursor

@param aBitmapHandle The handle to the cursor bitmap
@param aMaskHandle The handle to the cursor mask
@param aHotpointX The hot point X position on the cursor 
@param aHotpointY The hot point Y position on the cursor 
*/
RWsSprite* CVNCViewerScrollableContainer::CreatePixmapCursorL(TInt aBitmapHandle, TInt aMaskHandle, TInt aHotpointX, TInt aHotpointY)
	{	
	// Construct two new bitmaps
	iCursorBitmap = new(ELeave) CFbsBitmap();
	iCursorMask = new(ELeave) CFbsBitmap();

	// Duplicate the engine bitmaps
	User::LeaveIfError(iCursorBitmap->Duplicate(aBitmapHandle));
	User::LeaveIfError(iCursorMask->Duplicate(aMaskHandle));	
		
	// Get the window server
	RWsSession& wsSession = CEikonEnv::Static()->WsSession();
	
	// The construct the pointer
	iCursor = RWsSprite(wsSession);
	User::LeaveIfError(iCursor.Construct(*iVNCViewPortWindow->DrawableWindow(), iCursorPos, 0));

	// Append the bitmap
	TSpriteMember member;
	member.iBitmap = iCursorBitmap;
	member.iMaskBitmap = iCursorMask;
	member.iInvertMask = EFalse;
	member.iInterval = 0;
	member.iOffset = TPoint(-aHotpointX, -aHotpointY);
	User::LeaveIfError(iCursor.AppendMember(member));
	
	// Activate the pixmap
	User::LeaveIfError(iCursor.Activate());

	iCursorCreated = ETrue;
	
	return &iCursor;
	}

/**
From MVNCViewerEvent, The engine is asking the UI to move the cursor

@param aX The cursor X position
@param aY The cursor Y position
*/
void CVNCViewerScrollableContainer::WarpPointerL(TInt aX, TInt aY)
	{
	iCursorPos.iX = aX;
	iCursorPos.iY = aY;
	if (!iCursorLocated)
		{
		FindCursorL();
		iCursorLocated = ETrue;
		}

	TPoint point(iCursorPos);
	TranslateRemoteToLocalPoint(point);
	
	if (iCursorCreated)
		{
		// Get the window server
		iCursor.SetPosition(point);
		}
	}
	
/**
From MVNCViewerEvent, Free the cursor
*/
void CVNCViewerScrollableContainer::FreeCursorL()
	{
	if (iCursorCreated)
		iCursor.Close();
	}
	
/**
From MVNCViewerEvent, The server is sending a message

@param The server message
*/
void CVNCViewerScrollableContainer::ServerMessageL(const TDesC& aMessage)
	{
	TPtrC messageTitle(CEikonEnv::Static()->AllocReadResourceAsDes16LC(R_VNCVIEWER_SERVER_MESSAGE)->Des());
	CEikonEnv::Static()->InfoWinL(messageTitle, aMessage);
	CleanupStack::PopAndDestroy(); // messageTitle	
	
	// We've had an error
	iView.DestroyEngine(ETrue);
	}

/**
From MSendText, The send text dialog has closed, send the data on

@param aErr The status
@param aText The text to send
*/
void CVNCViewerScrollableContainer::SendTextL(TInt aErr, const TDesC& aText)
	{
	if (KErrNone == aErr)
		{
		iVNCViewMsgProc->SendTextL(aText);
		}

	delete iSendTextActive;
	iSendTextActive = NULL;
	}

/**
From MScale, The scale has changed

@param aCmd The command pressed
@param aScale The scale
*/
void CVNCViewerScrollableContainer::ScaleL(TInt aCmd, const TReal aScale)
	{
	if (EVNCViewerScaleCmdDone == aCmd)
		{
		SetScaleL(aScale);
		}

	delete iScaleActive;
	iScaleActive = NULL;
	}

/**
Set the arrow key mode

@param aOn Arrow controls cursor?
*/
void CVNCViewerScrollableContainer::SetArrowKeysControlCursor(TBool aOn)
	{
	iArrowKeypressMovesPointer = aOn;
	}
	
/**
Open the send text dialog
*/
void CVNCViewerScrollableContainer::SendTextDialogL()
	{
	if (!iSendTextActive)
		iSendTextActive = CSendTextActive::NewL(*this);
	}

/**
Open the scale dialog
*/
void CVNCViewerScrollableContainer::SetCustomScaleL()
	{
	if (!iScaleActive)
		iScaleActive = CScaleActive::NewL(*this, iScale);
	}

/**
Set the Mouse button mode (left, middle or right)

@param aButtonMode The mouse button mode
*/
void CVNCViewerScrollableContainer::SetButtonMode(TButtonMode aButtonMode)
	{
	iButtonMode = aButtonMode;	
	}

/**
Refresh the screen
*/
void CVNCViewerScrollableContainer::RefreshL()
	{
	iVNCViewMsgProc->RefreshL();
	}

/**
Set the scale factor

@param aScale The new scale
*/
void CVNCViewerScrollableContainer::SetScaleL(TReal aScale)
	{
	// Get the current cursor point
	TPoint cursorPoint = iCursorPos;
	TranslateLocalToRemotePoint(cursorPoint);
	
	iScale = aScale;
	if (iScale == fullscreenScale)
		{
		// We don't need scroll bars
		ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff,
	                                                CEikScrollBarFrame::EOff);
		
		iVNCViewPortWindow->SetRect(Rect());	
		iVNCViewMsgProc->SetFullscreen(ETrue);
		}
	else
		{
		// Scale works in the opposite way using the scrollable container
		if (iScale != absoluteScale)
			iScale = 1 - iScale;
		
		TRect bitmapRect(TPoint(0, 0), iMainBitmap->SizeInPixels());
		TRect viewportRect = ScaledRect(bitmapRect);
		iVNCViewPortWindow->SetRect(viewportRect);
		iVNCViewMsgProc->SetFullscreen(EFalse);
		
		// We need scroll bars
		ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto,
	                                                CEikScrollBarFrame::EAuto);
		}

	// Update the scroll bar state
	SetPageSize(MinimumPageSize());
	UpdateScrollBarsL();
	FindCursorL();
	
	// Move the pointer
	WarpPointerL(cursorPoint.iX, cursorPoint.iY);
	
	// Redraw
	DrawNow();
	}

/**
Start the process of connecting to the VNC server
*/
void CVNCViewerScrollableContainer::ConnectL()
	{
	// Create the engine
	TRAPD(err, iDocument.SetupCmdParamsL());
	if (KErrNone == err)
		{
		TRAPD(err, iVNCViewMsgProc = CVNCViewMsgProcessor::NewL(*this, iDocument.CmdParams()));
		if (KErrNone != err)
			{
			CEikonEnv::Static()->InfoWinL(R_VNCVIEWER_NOENGINE_TITLE, R_VNCVIEWER_NOENGINE);
			}
		
		// Store the fullscreen
		iScale = iDocument.Settings().iFullScreen ? fullscreenScale : 0;
		}
	else
		{
		// We've had an error
		iView.DestroyEngine(ETrue);	
		}
	}

/**
Forward on any foreground events

@param aForeground Foreground or Background?
*/
void CVNCViewerScrollableContainer::HandleForegroundEventL(TBool aForeground)
	{
	if (iLoggedOn)
		iVNCViewMsgProc->HandleForegroundEventL(aForeground);	
	}

/**
Locate the cursor
*/
void CVNCViewerScrollableContainer::FindCursorL()
	{
	if (iDocument.Settings().iCursor)
		{
		TPoint point = iCursorPos;
		TranslateRemoteToLocalPoint(point);

		// Get the page size
		TSize pageSize = Size();
		
		point.iX -= pageSize.iWidth / 2;
		point.iY -= pageSize.iHeight / 2;
		if (point.iX < 0)
			point.iX = 0;	
		if (point.iY < 0)
			point.iY = 0;	
		
		ScrollTo(point);
		UpdateScrollBarsL();
		}
	}

/**
The UI configuration has changed
*/
void CVNCViewerScrollableContainer::SizeChanged()
	{
	iVNCViewPortWindow->SetSize(Size());
	
	// Update the scroll bar state
	SetPageSize(MinimumPageSize());
	UpdateScrollBarsL();

	DrawNow();
	}

/**
Handle any pointer events

@param aPointerEvent The pointer event
*/
void CVNCViewerScrollableContainer::HandlePointerEventL(const TPointerEvent& aPointerEvent)
	{
	if (!iLoggedOn)
		return;
	
	TPointerEvent event = aPointerEvent;
	
	// Alter the pointer event for middle and right hand buttons
	if (iButtonMode == EButtonMiddle)
		{
		if (event.iType == TPointerEvent::EButton1Down)
			event.iType = TPointerEvent::EButton2Down;
		else if (event.iType == TPointerEvent::EButton1Up)
			event.iType = TPointerEvent::EButton2Up;
		}
	if (iButtonMode == EButtonRight)
		{
		if (event.iType == TPointerEvent::EButton1Down)
			event.iType = TPointerEvent::EButton3Down;
		else if (event.iType == TPointerEvent::EButton1Up)
			event.iType = TPointerEvent::EButton3Up;
		}
		
	if (iScale != absoluteScale)
		TranslateLocalToRemotePoint(event.iPosition);
	
	if (iScale == fullscreenScale)
		iVNCViewMsgProc->SendMouseEventL(event);
	else
		{
		// We need to send the event on
		TSize viewportSize = Size();

		iVNCViewMsgProc->SendMouseEventL(event);
		
		if (aPointerEvent.iType == TPointerEvent::EDrag)
			{
			// Check if we should move the scrollbars
			TPoint scrollOffset = ScrollOffset();

			TPoint offset = aPointerEvent.iPosition - scrollOffset;
		
			// Check if the pointer is at the left hand edge
			if (offset.iX < scrollBumpMargin)
				{
				scrollOffset.iX -= scrollBump;
				ScrollTo(scrollOffset);
				UpdateScrollBarsL();
				}

			// Check if the pointer is at the top edge
			if (offset.iY < scrollBumpMargin)
				{
				scrollOffset.iY -= scrollBump;
				ScrollTo(scrollOffset);
				UpdateScrollBarsL();
				}

			CEikScrollBarFrame* scrollBarFrame = ScrollBarFrame();
			TInt scrollSize = scrollBarFrame->ScrollBarBreadth(CEikScrollBar::EVertical);
			
			// Check if the pointer is at the right hand edge
			if (offset.iX > (viewportSize.iWidth - scrollSize - scrollBumpMargin))
				{
				scrollOffset.iX += scrollBump;
				ScrollTo(scrollOffset);
				UpdateScrollBarsL();
			}

			scrollSize = scrollBarFrame->ScrollBarBreadth(CEikScrollBar::EHorizontal);

			// Check if the pointer is at the bottom edge
			if (offset.iY > (viewportSize.iHeight - scrollSize - scrollBumpMargin))
				{
				scrollOffset.iY += scrollBump;
				ScrollTo(scrollOffset);
				UpdateScrollBarsL();
				}
			}
		}
	}

/**
Handle any keyboard events

@param aKeyEvent The key event
@param aType The key event type
@return TKeyResponse Whether the event was used
*/
TKeyResponse CVNCViewerScrollableContainer::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
	{
	if (!iLoggedOn)
		return EKeyWasNotConsumed;
	
	TInt keytype = 0;

	TConnectionSettings& settings = iDocument.Settings(); 

	// Set the message type
	switch(aType)
		{
		case EEventKey:
			keytype = EKey;
			break;
		case EEventKeyUp:
			keytype = EKeyUp;
			break;
		case EEventKeyDown:
			keytype = EKeyDown;
			break;
		default:
			return EKeyWasNotConsumed;
		}

	// Calculate the state of the modifiers
	TInt modifiers = 0;

	CQikCommandManager& manager = CQikCommandManager::Static();
	if (manager.Command(*this, EVNCViewerModifierShift)->Checked() || aKeyEvent.iModifiers & EModifierShift)
		modifiers |= EKeyModShift;
	if (manager.Command(*this, EVNCViewerModifierCtrl)->Checked() || aKeyEvent.iModifiers & EModifierCtrl)
		modifiers |= EKeyModCtrl;
	if (manager.Command(*this, EVNCViewerModifierAlt)->Checked() || aKeyEvent.iModifiers & EModifierAlt)
		modifiers |= EKeyModAlt;
	if (manager.Command(*this, EVNCViewerModifierFunc)->Checked() || aKeyEvent.iModifiers & EModifierFunc)
		modifiers |= EKeyModFunc;
	
	if (aKeyEvent.iRepeats > 0)
		++iPointerAcceleration;
	else
		iPointerAcceleration = 0;

	TInt keyCode = aKeyEvent.iCode;
	
	TUint16 keyToSend = 0;
		
	switch(keyCode)
		{
		case EKeyBackspace:
			keyToSend = Keysyms::KKeysymBackSpace;
			break;		
		case EKeyTab:
			keyToSend = Keysyms::KKeysymTab;
			break;		
		case EKeyEnter:
			keyToSend = Keysyms::KKeysymReturn;
			break;		
		case EKeyEscape:
			keyToSend = Keysyms::KKeysymEscape;
			break;		
		case EKeyDelete:
			keyToSend = Keysyms::KKeysymDelete;
			break;		
		case EKeyPrintScreen:
			keyToSend = Keysyms::KKeysymPrint;
			break;		
		case EKeyHome:
			keyToSend = Keysyms::KKeysymHome;
			break;		
		case EKeyEnd:
			keyToSend = Keysyms::KKeysymEnd;
			break;		
		case EKeyPageUp:
			keyToSend = Keysyms::KKeysymPage_Up;
			break;		
		case EKeyPageDown:
			keyToSend = Keysyms::KKeysymPage_Down;
			break;		
		case EKeyInsert:
			keyToSend = Keysyms::KKeysymInsert;
			break;		
		case EKeyLeftArrow:
		case EDeviceKeyFourWayLeft:
			if ((!settings.iCursor) && (iScale != fullscreenScale))
				{
				Scroll(-scrollBump, 0);
				return EKeyWasConsumed;
				}
		
			if (iArrowKeypressMovesPointer)
				{
				TBool scroll = CursorInScreenArea();
				iCursorPos.iX -= (moveCursor + iPointerAcceleration);
				if (iCursorPos.iX < 0)
					iCursorPos.iX = 0;
				MimicPointerMoveL(scroll);
				return EKeyWasConsumed;
				}
			else
				keyToSend = Keysyms::KKeysymLeft;
			break;		
		case EKeyRightArrow:
		case EDeviceKeyFourWayRight:
			if ((!settings.iCursor) && (iScale != fullscreenScale))
				{
				Scroll(scrollBump, 0);
				return EKeyWasConsumed;
				}

			if (iArrowKeypressMovesPointer)
				{
				TBool scroll = CursorInScreenArea();
				iCursorPos.iX += (moveCursor + iPointerAcceleration);
				TSize sizeBitmap = iMainBitmap->SizeInPixels();
				if (iCursorPos.iX > sizeBitmap.iWidth - 1)
					iCursorPos.iX = sizeBitmap.iWidth - 1;
				MimicPointerMoveL(scroll);
				return EKeyWasConsumed;
				}
			else
				keyToSend = Keysyms::KKeysymRight;
			break;		
		case EKeyUpArrow:
		case EDeviceKeyFourWayUp:
			if ((!settings.iCursor) && (iScale != fullscreenScale))
				{
				Scroll(0, -scrollBump);
				return EKeyWasConsumed;
				}

			if (iArrowKeypressMovesPointer)
				{
				TBool scroll = CursorInScreenArea();
				iCursorPos.iY -= (moveCursor + iPointerAcceleration);
				if (iCursorPos.iY < 0)
					iCursorPos.iY = 0;
				MimicPointerMoveL(scroll);
				return EKeyWasConsumed;
				}
			else
				keyToSend = Keysyms::KKeysymUp;
			break;		
		case EKeyDownArrow:
		case EDeviceKeyFourWayDown:
			if ((!settings.iCursor) && (iScale != fullscreenScale))
				{
				Scroll(0, scrollBump);
				return EKeyWasConsumed;
				}

			if (iArrowKeypressMovesPointer)
				{
				TBool scroll = CursorInScreenArea();
				iCursorPos.iY += (moveCursor + iPointerAcceleration);
				TSize sizeBitmap = iMainBitmap->SizeInPixels();
				if (iCursorPos.iY > sizeBitmap.iHeight - 1)
					iCursorPos.iY = sizeBitmap.iHeight - 1;
				MimicPointerMoveL(scroll);
				return EKeyWasConsumed;
				}
			else
				keyToSend = Keysyms::KKeysymDown;
			break;		
		case EKeyLeftShift:
			keyToSend = Keysyms::KKeysymShift_L;
			modifiers = 0;
			break;		
		case EKeyRightShift:
			keyToSend = Keysyms::KKeysymShift_R;
			modifiers = 0;
			break;		
		case EKeyLeftAlt:
			keyToSend = Keysyms::KKeysymAlt_L;
			modifiers = 0;
			break;		
		case EKeyRightAlt:
			keyToSend = Keysyms::KKeysymAlt_R;
			modifiers = 0;
			break;		
		case EKeyLeftCtrl:
			keyToSend = Keysyms::KKeysymControl_L;
			modifiers = 0;
			break;		
		case EKeyRightCtrl:
			keyToSend = Keysyms::KKeysymControl_R;
			modifiers = 0;
			break;		
		case EKeyLeftFunc:
			keyToSend = Keysyms::KKeysymMeta_L;
			modifiers = 0;
			break;		
		case EKeyRightFunc:
			keyToSend = Keysyms::KKeysymMeta_R;
			modifiers = 0;
			break;		
		case EKeyF1:
			keyToSend = Keysyms::KKeysymF1;
			break;		
		case EKeyF2:
			keyToSend = Keysyms::KKeysymF2;
			break;		
		case EKeyF3:
			keyToSend = Keysyms::KKeysymF3;
			break;		
		case EKeyF4:
			keyToSend = Keysyms::KKeysymF4;
			break;		
		case EKeyF5:
			keyToSend = Keysyms::KKeysymF5;
			break;		
		case EKeyF6:
			keyToSend = Keysyms::KKeysymF6;
			break;		
		case EKeyF7:
			keyToSend = Keysyms::KKeysymF7;
			break;		
		case EKeyF8:
			keyToSend = Keysyms::KKeysymF8;
			break;		
		case EKeyF9:
			keyToSend = Keysyms::KKeysymF9;
			break;		
		case EKeyF10:
			keyToSend = Keysyms::KKeysymF10;
			break;		
		case EKeyF11:
			keyToSend = Keysyms::KKeysymF11;
			break;		
		case EKeyF12:
			keyToSend = Keysyms::KKeysymF12;
			break;		
		default:
			if((aKeyEvent.iCode >= 0x20 && aKeyEvent.iCode <= 0x7e))
				keyToSend = aKeyEvent.iCode;
			else
				return EKeyWasNotConsumed;
		}

	// Send the keypress
	iVNCViewMsgProc->SendKeyEventL(keytype, keyToSend, modifiers);
		
	return EKeyWasConsumed;
	}

/**
Mimic a pointer move

@param aScroll Whether to scroll the window
*/
void CVNCViewerScrollableContainer::MimicPointerMoveL(TBool aScroll)
	{
	TPointerEvent pointerEvent;
	pointerEvent.iType = iPointerDown ? TPointerEvent::EDrag : TPointerEvent::EMove;
	pointerEvent.iPosition = iCursorPos; 
	iVNCViewMsgProc->SendMouseEventL(pointerEvent);

	if ((iScale != fullscreenScale) && (aScroll))
		{
		// Check if we should move the scrollbars
		TPoint scrollOffset = ScrollOffset();

		TPoint offset = pointerEvent.iPosition - scrollOffset;

		TInt scrollAmount = moveCursor + iPointerAcceleration;

		TSize viewportSize = ViewPortSize();
	
		// Check if the pointer is at the left hand edge
		if (offset.iX < scrollBumpMargin)
			{
			scrollOffset.iX -= scrollAmount;
			ScrollTo(scrollOffset);
			UpdateScrollBarsL();
			}

		// Check if the pointer is at the top edge
		if (offset.iY < scrollBumpMargin)
			{
			scrollOffset.iY -= scrollAmount;
			ScrollTo(scrollOffset);
			UpdateScrollBarsL();
			}
		
		// Check if the pointer is at the right hand edge
		if (offset.iX > (viewportSize.iWidth - scrollBumpMargin))
			{
			scrollOffset.iX += scrollAmount;
			ScrollTo(scrollOffset);
			UpdateScrollBarsL();
			}

		// Check if the pointer is at the bottom edge
		if (offset.iY > (viewportSize.iHeight - scrollBumpMargin))
			{
			scrollOffset.iY += scrollAmount;
			ScrollTo(scrollOffset);
			UpdateScrollBarsL();
			}
		}
	}

/**
Mimic a pointer up or down event

@param aDown Pointer Down or Up Event?
*/
void CVNCViewerScrollableContainer::MimicPointerDownUpL(TBool aDown)
	{
	iPointerDown = aDown;
	
	TPointerEvent pointerEvent;
	
	switch(iButtonMode)
		{
		case EButtonLeft:
			pointerEvent.iType = aDown ? TPointerEvent::EButton1Down : TPointerEvent::EButton1Up;
			break;
		case EButtonMiddle:
			pointerEvent.iType = aDown ? TPointerEvent::EButton2Down : TPointerEvent::EButton2Up;
			break;
		case EButtonRight:
			pointerEvent.iType = aDown ? TPointerEvent::EButton3Down : TPointerEvent::EButton3Up;
			break;
		}
		
	pointerEvent.iPosition.iX = iCursorPos.iX; 
	pointerEvent.iPosition.iY = iCursorPos.iY; 
	iVNCViewMsgProc->SendMouseEventL(pointerEvent);
	}

/**
Test whether the cursor is visible

@return TBool Whether the cursor is visible
*/
TBool CVNCViewerScrollableContainer::CursorInScreenArea()
	{
	if (iScale == fullscreenScale)
		return ETrue;
	else
		{
		// We need to send the event on
		TSize viewportSize = ViewPortSize();
			
		TPoint scrollOffset = ScrollOffset();
	
		if ((iCursorPos.iX > scrollOffset.iX) &&
			(iCursorPos.iX < scrollOffset.iX + viewportSize.iWidth) &&
			(iCursorPos.iY > scrollOffset.iY) &&
			(iCursorPos.iY < scrollOffset.iY + viewportSize.iHeight))
			{
			return ETrue;
			}
		else
			{
			return EFalse;	
			}
		}
	}

/**
Return the size of the viewable window

@return TSize The size
*/
TSize CVNCViewerScrollableContainer::ViewPortSize()
	{
	TSize viewportSize = Rect().Size();
	CEikScrollBarFrame* sbFrame = ScrollBarFrame();
	if (sbFrame->HScrollBarVisibility())
		viewportSize.iWidth -= sbFrame->ScrollBarBreadth(CEikScrollBar::EHorizontal);
	if (sbFrame->VScrollBarVisibility())
		viewportSize.iHeight -= sbFrame->ScrollBarBreadth(CEikScrollBar::EVertical);
	return viewportSize;			
	}
	
/**
Translate a local coordinate to the remote (VNC server) coordinate

@param aPoint The point to translate
@param aRelative Whether to take into account the relative coordinates of the top-left screen
*/
void CVNCViewerScrollableContainer::TranslateLocalToRemotePoint(TPoint& aPoint)
	{
	if (iScale == absoluteScale)
		return;
	
	// We need to coords
	TSize viewportSize = iVNCViewPortWindow->Rect().Size();

	// Re-scale to coords
	TSize bitmapSize = iMainBitmap->SizeInPixels();
		
	aPoint.iX = (aPoint.iX * bitmapSize.iWidth) / viewportSize.iWidth;
	aPoint.iY = (aPoint.iY * bitmapSize.iHeight) / viewportSize.iHeight;
	}

/**
Translate a remote (VNC server) coordinate to the local coordinate

@param aPoint The point to translate
@param aRelative Whether to take into account the relative coordinates of the top-left screen
*/
void CVNCViewerScrollableContainer::TranslateRemoteToLocalPoint(TPoint& aPoint)
	{
	if (iScale == absoluteScale)
		return;
	
	// We need to coords
	TSize viewportSize = iVNCViewPortWindow->Rect().Size();

	// Re-scale to coords
	TSize bitmapSize = iMainBitmap->SizeInPixels();
		
	aPoint.iX = (aPoint.iX * viewportSize.iWidth) / bitmapSize.iWidth;
	aPoint.iY = (aPoint.iY * viewportSize.iHeight) / bitmapSize.iHeight;
	}

/**
Scale a rectangle using the current scale

@param aRect The unscaled rectangle
@return TRect The scaled rectangle
*/
TRect CVNCViewerScrollableContainer::ScaledRect(const TRect& aRect)
	{
	if (iScale == absoluteScale)
		return aRect;
	
	TPoint rect_point(static_cast<TInt>(aRect.iTl.iX * iScale), static_cast<TInt>(aRect.iTl.iY * iScale));
	TSize rect_size(static_cast<TInt>(aRect.Width() * iScale), static_cast<TInt>(aRect.Height() * iScale));
	
	TRect scaledRect(rect_point, rect_size);
	return scaledRect;	
	}

/**
Handle key events via a menu option

@param aKey The key to send
*/
TBool CVNCViewerScrollableContainer::SendKeyL(TInt aKey)
	{	
	// Calculate the state of the modifiers
	TInt modifiers = 0;

	CQikCommandManager& manager = CQikCommandManager::Static();
	if (manager.Command(*this, EVNCViewerModifierShift)->Checked())
		modifiers |= EKeyModShift;
	if (manager.Command(*this, EVNCViewerModifierCtrl)->Checked())
		modifiers |= EKeyModCtrl;
	if (manager.Command(*this, EVNCViewerModifierAlt)->Checked())
		modifiers |= EKeyModAlt;
	if (manager.Command(*this, EVNCViewerModifierFunc)->Checked())
		modifiers |= EKeyModFunc;
	
	TInt keycode = 0;
	
	switch(aKey)
		{
		case EVNCViewerKeyEnter:
			keycode = Keysyms::KKeysymReturn;
			break;
		case EVNCViewerKeyBackspace:
			keycode = Keysyms::KKeysymBackSpace;
			break;
		case EVNCViewerKeyDelete:
			keycode = Keysyms::KKeysymDelete;
			break;
		case EVNCViewerKeyInsert:
			keycode = Keysyms::KKeysymInsert;
			break;
		case EVNCViewerKeyTab:
			keycode = Keysyms::KKeysymTab;
			break;
		case EVNCViewerKeyEscape:
			keycode = Keysyms::KKeysymEscape;
			break;
		case EVNCViewerKeyLeft:
			keycode = Keysyms::KKeysymLeft;
			break;
		case EVNCViewerKeyRight:
			keycode = Keysyms::KKeysymRight;
			break;
		case EVNCViewerKeyUp:
			keycode = Keysyms::KKeysymUp;
			break;
		case EVNCViewerKeyDown:
			keycode = Keysyms::KKeysymDown;
			break;
		case EVNCViewerKeyHome:
			keycode = Keysyms::KKeysymHome;
			break;
		case EVNCViewerKeyEnd:
			keycode = Keysyms::KKeysymEnd;
			break;
		case EVNCViewerKeyPgUp:
			keycode = Keysyms::KKeysymPage_Up;
			break;
		case EVNCViewerKeyPgDn:
			keycode = Keysyms::KKeysymPage_Down;
			break;
		case EVNCViewerKeyF1:
			keycode = Keysyms::KKeysymF1;
			break;
		case EVNCViewerKeyF2:
			keycode = Keysyms::KKeysymF2;
			break;
		case EVNCViewerKeyF3:
			keycode = Keysyms::KKeysymF3;
			break;
		case EVNCViewerKeyF4:
			keycode = Keysyms::KKeysymF4;
			break;
		case EVNCViewerKeyF5:
			keycode = Keysyms::KKeysymF5;
			break;
		case EVNCViewerKeyF6:
			keycode = Keysyms::KKeysymF6;
			break;
		case EVNCViewerKeyF7:
			keycode = Keysyms::KKeysymF7;
			break;
		case EVNCViewerKeyF8:
			keycode = Keysyms::KKeysymF8;
			break;
		case EVNCViewerKeyF9:
			keycode = Keysyms::KKeysymF9;
			break;
		case EVNCViewerKeyF10:
			keycode = Keysyms::KKeysymF10;
			break;
		case EVNCViewerKeyF11:
			keycode = Keysyms::KKeysymF11;
			break;
		case EVNCViewerKeyF12:
			keycode = Keysyms::KKeysymF12;
			break;
		case EVNCViewerKeyPrintScreen:
			keycode = Keysyms::KKeysymPrint;
			break;
		case EVNCViewerKeyPause:
			keycode = Keysyms::KKeysymPause;
			break;
		case EVNCViewerCtrlAltDel:
			keycode = Keysyms::KKeysymDelete;
			modifiers = EKeyModCtrl | EKeyModAlt;
			break;
		case EVNCViewerClick:
			if (iArrowKeypressMovesPointer)
				{
				iPointerAcceleration = 0;
				if (modifiers & EKeyModCtrl)
					{
					if (!iPointerDown)
						MimicPointerDownUpL(ETrue);
					else
						MimicPointerDownUpL(EFalse);
					}
				else
					{
					MimicPointerDownUpL(ETrue);
					MimicPointerDownUpL(EFalse);
					}
				return EKeyWasConsumed;
				}
			else
				return EKeyWasNotConsumed;
			break;		
		}

	// Send the keypress
	iVNCViewMsgProc->SendKeyEventL(EKey, keycode, modifiers);
	return (keycode != 0);
	}

/**
Post a quit message to the engine
*/
TInt CVNCViewerScrollableContainer::PostQuit()
	{
	if (iVNCViewMsgProc)
		{
		iVNCViewMsgProc->PostQuit();
		return ETrue;
		}
	else 
		return EFalse;
	}

/**
Draw the background

@param aRect The bounding rectangle
*/
void CVNCViewerScrollableContainer::Draw(const TRect& aRect) const
	{
	CWindowGc& gc = SystemGc();	
	gc.Clear(aRect);
	}

//---------------------------------------------------------------
//---------------------------------------------------------------



/**
Creates and constructs the viewport

@param aViewContainer Reference to the container
@return CVNCViewportWindow* Pointer to a CVNCViewportWindow object
*/
CVNCViewportWindow* CVNCViewportWindow::NewL(CVNCViewerScrollableContainer& aViewContainer)
	{
	CVNCViewportWindow* self = new(ELeave) CVNCViewportWindow(aViewContainer);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
	}

/**
Destructor for the viewport
*/
CVNCViewportWindow::~CVNCViewportWindow()
	{
	}

/**
Creates and constructs the viewport

@param aViewContainer Reference to the container
*/
CVNCViewportWindow::CVNCViewportWindow(CVNCViewerScrollableContainer& aViewport)
: iViewport(aViewport)
	{
	}
	
/**
2nd stage construction of the container.
*/
void CVNCViewportWindow::ConstructL()
	{
	SetContainerWindowL(iViewport);
	ActivateL();
	}

/**
Draw the bitmap into the window

@param aRect The bounding rectangle
*/
void CVNCViewportWindow::Draw(const TRect& aRect) const
	{
	if (iViewport.LoggedOn())
		{
		CWindowGc& gc = SystemGc();	

	  	gc.SetClippingRect(aRect);
		gc.DrawBitmap(Rect(), iViewport.Bitmap());
		gc.CancelClippingRect();
		}
	}

/**
Handle a pointer event

@param aPointerEvent The pointer event
*/
void CVNCViewportWindow::HandlePointerEventL(const TPointerEvent& aPointerEvent)
	{
	iViewport.HandlePointerEventL(aPointerEvent);
	}
	

