/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

#include "VncIpc.h"

const char* KSysPath = "c:\\system\\vncviewer\\";

/**
Constructor for the IPC mechanism
@param aServer Whether this is the UI server or client
@return Pointer to a CVncIpc object
*/
EXPORT_C CVncIpc* CVncIpc::NewL(TBool aServer)
	{
	CVncIpc* ipc = new(ELeave) CVncIpc(aServer);
	CleanupStack::PushL(ipc);
	ipc->ConstructL();
	CleanupStack::Pop(ipc); // Ipc
	return ipc;
	}
	
/**
Destructor for the IPC mechanism
*/
EXPORT_C CVncIpc::~CVncIpc()
	{		
	// Need to close the mechanisms when the API is available
	if (iEventFifo != -1)		
		close(iEventFifo);
	if (iToUIFifo != -1)		
		close(iToUIFifo);
	if (iFromUIFifo != -1)		
		close(iFromUIFifo);
	if (iQuitFifo != -1)		
		close(iQuitFifo);
	
	if (iUI)
		{
		unlink("vnc_event_fifo");
		unlink("vnc_to_ui_fifo");
		unlink("vnc_from_ui_fifo");
		unlink("vnc_quit_fifo");
		}
		
	// Destroy the semaphore
	semun arg;
	semctl(iSem, 0, IPC_RMID, arg);

	// Close the file
	if (iFile)	
		close(iFile);
	
	// Remove the director
	rmdir(KSysPath);
	}

/**
Constructor for the IPC mechanism
@param aServer Whether this is the UI server or client
*/
CVncIpc::CVncIpc(TBool aUI)
: iUI(aUI), iEventFifo(-1), iToUIFifo(-1), iFromUIFifo(-1)
	{
	iMsgSize = sizeof(vncmsg);
	}

/**
2nd phase constructor
*/	
void CVncIpc::ConstructL()
	{
	mode_t accessMode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;

	// Create a path to use and change to it
	int ret = mkdir(KSysPath, accessMode);
	if ((ret == -1) && (errno != EEXIST))
		User::Leave(KErrArgument);
	ret = chdir(KSysPath);
	if (ret == -1)
		User::Leave(KErrArgument);
	
	// Construct the 
	iFile = open("Vncview", O_RDWR | O_CREAT);
	if (!iFile)
		User::Leave(KErrNotFound);
		
	// Construct the keys
	iKeySem = ftok("Vncview", 1);
	if (iKeySem == -1)
		User::Leave(KErrArgument);
	
	// Get a semaphore
	iSem = semget(iKeySem, 1, IPC_CREAT | accessMode);
	if (iSem == -1)
		User::Leave(KErrArgument);
	
	// Construct the fifos
	ret = mkfifo("c:\\system\\vncviewer\\vnc_event_fifo", accessMode);
	if ((ret == -1) && (errno != EEXIST))
		User::Leave(KErrArgument);
	ret = mkfifo("c:\\system\\vncviewer\\vnc_to_ui_fifo", accessMode);
	if ((ret == -1) && (errno != EEXIST))
		User::Leave(KErrArgument);
	ret = mkfifo("c:\\system\\vncviewer\\vnc_from_ui_fifo", accessMode);
	if ((ret == -1) && (errno != EEXIST))
		User::Leave(KErrArgument);
	ret = mkfifo("c:\\system\\vncviewer\\vnc_quit_fifo", accessMode);
	if ((ret == -1) && (errno != EEXIST))
		User::Leave(KErrArgument);
	
	sembuf SemBuf;
	
	// Construct the relevant ends of the fifo pipes
	if (iUI)
		{
		iEventFifo = open("vnc_event_fifo", O_WRONLY); 		
		if (iEventFifo == -1)
			User::Leave(KErrArgument);
		iToUIFifo = open("vnc_to_ui_fifo", O_RDONLY); 		
		if (iToUIFifo == -1)
			User::Leave(KErrArgument);
		iFromUIFifo = open("vnc_from_ui_fifo", O_WRONLY); 		
		if (iFromUIFifo == -1)
			User::Leave(KErrArgument);
		iQuitFifo = open("vnc_quit_fifo", O_RDONLY); 		
		if (iQuitFifo == -1)
			User::Leave(KErrArgument);
		
		// Wait for engine IPC to start
		SemBuf.sem_num = 0;
		SemBuf.sem_op = 1;
		SemBuf.sem_flg = 0;
		
		if (-1 == semop(iSem, &SemBuf, 1))
			User::Leave(KErrArgument);
		}
	else
		{
		iEventFifo = open("vnc_event_fifo", O_RDONLY); 		
		if (iEventFifo == -1)
			User::Leave(KErrArgument);
		iToUIFifo = open("vnc_to_ui_fifo", O_WRONLY); 		
		if (iToUIFifo == -1)
			User::Leave(KErrArgument);
		iFromUIFifo = open("vnc_from_ui_fifo", O_RDONLY); 		
		if (iFromUIFifo == -1)
			User::Leave(KErrArgument);
		iQuitFifo = open("vnc_quit_fifo", O_WRONLY); 		
		if (iQuitFifo == -1)
			User::Leave(KErrArgument);
		
		// Signal engine IPC has started
		SemBuf.sem_num = 0;
		SemBuf.sem_op = -1;
		SemBuf.sem_flg = 0;
		
		if (-1 == semop(iSem, &SemBuf, 1))
			User::Leave(KErrArgument);
		}
	}

/**
Get the event Fifo
@return int The event fifo
*/
EXPORT_C int CVncIpc::EventFifo()
	{
	return iEventFifo;
	}

/**
Wait for data to arrive
@param aEventType The message type
@param aEvent The message contents
*/
EXPORT_C CVncIpc::TWaitPipe CVncIpc::WaitForData(int aStdErr)
	{
	fd_set set;
	FD_ZERO(&set);

	int fd = iUI ? iToUIFifo : iFromUIFifo;
	int maxfd = fd;
	
	// The normal pipe
	FD_SET(fd, &set);
	
	// The Stderr pipe
	FD_SET(aStdErr, &set);
	if (aStdErr > maxfd)
		maxfd = aStdErr;
	
	// The quit pipe
	FD_SET(iQuitFifo, &set);
	if (iQuitFifo > maxfd)
		maxfd = iQuitFifo;
		
	select(maxfd + 1, &set, NULL, NULL, NULL);
	if (FD_ISSET(fd, &set))
		return EWaitPipeNormal;
	if (FD_ISSET(aStdErr, &set))
		return EWaitPipeStdErr;
	if (FD_ISSET(iQuitFifo, &set))
		return EWaitPipeQuit;
	return EWaitPipeQuit;
	}

/**
Trigger the quit event
*/
EXPORT_C void CVncIpc::Quit()
	{
	int quit = 1;
	write(iQuitFifo, &quit, sizeof(int));
	}

/**
Send an event to the engine
@param aEventType The message type
@param aEvent The message contents
*/
EXPORT_C void CVncIpc::SendEventL(TInt aEventType, vncmsg& aEvent)
	{
	aEvent.iMsgType = aEventType;
	ssize_t bytes = Send(iEventFifo, &aEvent, iMsgSize);
	if (bytes == -1)
		User::Leave(KErrArgument);
	}
		
/**
Send a message to the other side
@param aMsgType The message type
@param aMessage The message contents
@param aExtraData Any extra data
@param aDataLength Extra data length
*/
EXPORT_C void CVncIpc::SendMessageL(TInt aMsgType, vncmsg& aMessage, char* aExtraData, TInt aDataLength)
	{
	aMessage.iMsgType = aMsgType;
	ssize_t bytes = Send(iUI ? iFromUIFifo : iToUIFifo, &aMessage, iMsgSize);
	if (bytes == -1)
		User::Leave(KErrArgument);
		
	if (aExtraData && aDataLength)
		{
		bytes = Send(iUI ? iFromUIFifo : iToUIFifo, aExtraData, aDataLength);
		if (bytes == -1)
			User::Leave(KErrArgument);
		}
	}

/**
Receive an event
@param aEvent The event contents
*/
EXPORT_C void CVncIpc::RecvEventL(vncmsg& aEvent)
	{
	ssize_t bytes = Recv(iEventFifo, &aEvent, iMsgSize);
	if (bytes == -1)
		User::Leave(KErrArgument);
	}

/**
Receive a message from the other queue
@param aMessage The message contents
*/
EXPORT_C void CVncIpc::RecvMessageL(vncmsg& aMessage)
	{
	ssize_t bytes = Recv(iUI ? iToUIFifo : iFromUIFifo, &aMessage, iMsgSize);
	if (bytes == -1)
		User::Leave(KErrArgument);
	}

/**
Receive a message from the other queue
@param aExtraData Any extra data
@param aDataLength The data length
*/
EXPORT_C void CVncIpc::RecvExtraData(char* aExtraData, TInt aDataLength)
	{
	ssize_t bytes = Recv(iUI ? iToUIFifo : iFromUIFifo, aExtraData, aDataLength);
	if (bytes == -1)
		User::Leave(KErrArgument);
	}

/**
Send a message to the queue
@param aQueue The queue on which to send
@param aData The message contents
@param aSize The size of the message
*/
ssize_t CVncIpc::Send(int aFifo, void* aData, int aSize)
	{
	ssize_t bytes = write(aFifo, aData, aSize);
	return bytes;
	}
	
/**
Receive a message from the queue
@param aQueue The queue on which to receive
@param aData The message contents
@param aSize The size of the message
*/
ssize_t CVncIpc::Recv(int aFifo, void* aData, int aSize)
	{
	ssize_t nbytes = read(aFifo, aData, aSize);
	return nbytes;
	}

