/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

/*
 * x11Windows.h - The Symbian Window System
 */

#ifndef __X11Windows_H__
#define __X11Windows_H__

// Includes
#include "x11.h"
#include <w32std.h>
#include <badesca.h>
#include <utf.h>
#include "vncipc.h"
#include "x11parsers.h"
#include "x11jpegconvert.h"

// Class to store Application inputs
class TAppInput
	{
public:
	int						iSource;
	XtInputCallbackProc		iProc;
	};

// Class to store Application events
class TAppEvent
	{
public:
	EventMask				iEventMask;
	XtEventHandler			iProc;
	XtPointer				iClosure;
	};
	
// Class to store Symbian display information
class TSymbDisplayInfo
	{
public:
	TInt iDepth;
	TInt iBpp;
	TInt iClass;
	TInt iRedMask;
	TInt iGreenMask;
	TInt iBlueMask;
	TInt iScanLinePad;
	};

// Conversion between X11 and Symbian colors
#define SymbToXColor(col) ((col << 8) + col)
#define XToSymbColor(col) (col >> 8)

/*
 * Class to handle access to the window system, either directly or 
 * via IPC to the UIKON user interface
 */	
class CSymbianWindowSystem : public CBase
	{
public:
	// Constructor and destructor
	CSymbianWindowSystem();
	virtual ~CSymbianWindowSystem();

	// Internal initialization
	void ConstructL();

	// Initialize the application and parse the options
	Widget InitAppL(XtAppContext* app_context_return,
	    XrmOptionDescList	options,
	    Cardinal		num_options,
	    int*		argc_in_out,
	    String*		argv_in_out);

	// Copy options into a structure pointer to by base	
	void GetAppResourcesL(XtPointer base, 
	    XtResourceList 	resources,
	    Cardinal 		num_resources);
	
	// Add a list of actions to the stored application actions
	void AppAddActionsL(XtActionList actions,
	    Cardinal 		num_actions);
		
	// Get the current display
	Display* GetDisplay() { return &iDisplay; };
	
	// Get information about possible display modes
	XVisualInfo* GetVisualInfoL(long vinfo_mask,
	    XVisualInfo* 	vinfo_template,
    	int*			nitems_return);
	
	// Get information about possible bitmap modes
	XPixmapFormatValues* ListPixmapFormatsL(int* count_return);

	// Get some values from the system	
	void GetValuesL(Widget widget, VA_LIST aVaList);
	
	// Create a colormap or palette for a display mode
	Colormap CreateColormapL(Visual* visual, int alloc);

	// Query the colors in a colormap
	void QueryColors(Colormap colormap, XColor* defs_in_out, int ncolors);
	
	// Allocate a color in the colormap closest to a desired color
	void AllocColor(Colormap colormap, XColor* screen_in_out);

	// Allocate some memory from the heap
	TAny* AllocMemoryL(TInt aSize);
	
	// Free memory 
	void FreeMemory(TAny* aMemory);

	// Create a bitmap
	XImage* CreateImageL(Visual* visual, int depth, int width, int height, int bytes_per_line);
	
	// Get a password from a dialog
	char* GetPassL(const char* prompt);
	
	// Add an application input to the processing loop
	XtInputId AppAddInputL(int source,
	    XtPointer 		condition,
    	XtInputCallbackProc proc,
    	XtPointer 		closure);

	// Remove an application input from the processing loop
	void AppRemoveInputL(XtInputId input);

	// Process events
	void AppProcessEventL();

	// Add an event handler		
	void AddEventHandlerL(EventMask event_mask,
	    _XtBoolean nonmaskable,
	    XtEventHandler proc,
  		XtPointer closure);
		
	// Protect a bitmap while transferring bitmap data into it		
	void PrepareMemoryTransfer();

	// Unlock a bitmap's data
	void EndMemoryTransfer();

	// Get the address of the bitmap data
	char* MemoryBuffer();

	// Update the user interface with a bitmap
	void PutImageL(int x, int y, int width, int height);
	
	// Change a GC (only a foreground color)
	void ChangeGcL(int mask, XGCValues* values);

	// Fill a rectangle in the bitmap with the current foreground color
	void FillRectangleL(int x, int y, int width, int height);
	
	// Convert jpeg data into an image	
	void JpegConvertL(Visual* vis, int width, int height, char* data, long length);

	// Get access to the Jpeg Data
	char* JpegGetDataL();

	// Discard the Jpeg Data
	void JpegDiscardData();
	
	// Translate a keycode in textual form to a symbolic key
	KeySym Translate(const char* aKeyCode);

	// Get the current value of the clipboard	
	void GetSelectionValue(
    	Widget 		widget,
    	Atom 		selection,
    	Atom 		target,
	    XtSelectionCallbackProc callback,
   	 	XtPointer 		closure,
    	XtTime 		xttime);
	
	// Store the clipboard contents
	void StoreBytesL(const char* bytes, int nbytes);

	// Create a pixmap (or bitmap)	
	Pixmap CreatePixmapL(unsigned int width,
	    unsigned int	height,
    	unsigned int	depth);
	
	// Copy a pixmap area to another pixmap
	void CopyAreaL(
	    Drawable		src,
	    Drawable		dest,
	    int			src_x,
	    int			src_y,
	    unsigned int	width,
	    unsigned int	height,
	    int			dest_x,
	    int			dest_y);

	// Free a pixmap
	void FreePixmapL(
	    Pixmap	pixmap);
	
	// Create a bitmap with existing data
	Pixmap CreateBitmapWithDataL(int width, int height, int visdepth, unsigned char* data);

	// Draw a bitmap onto another bitmap
	void DrawBitmapL(Pixmap bitmap, Pixmap mask, int x, int y, int width, int height, int hotpointx, int hotpointy);

	// Create a bitmap from 1 bit data
	Pixmap CreateBitmapFromDataL(int width, int height, const char* data);

	// Create a pixmap cursor
	Cursor CreatePixmapCursorL(Pixmap source,
	    Pixmap		mask,
		unsigned int	x,
    	unsigned int	y);
	
	// Move a cursor
	void WarpPointerL(int dest_x, int dest_y);

	// Free a cursor
	void FreeCursorL(
	    Cursor	cursor);

private:
	// Setup the current display
	void SetupDisplayL();
	
	// Get the Symbian information for a display mode
	void GetDisplayInfo(TDisplayMode aDisplayMode, TSymbDisplayInfo& aDisplayInfo);
	
	// Add allocated memory to the memory watcher
	void AddMemory(TAny* aMemory);
	
	// Remove allocated memory from the memory watcher
	void RemoveMemory(TAny* aMemory);
	
	// Process an incoming IPC message	
	void ProcessIncomingMessageL();

	// Process a Graphics expose event	
	void ProcessGraphicsExposeEvent(struct vncmsg& aMsg);
	
	// Process a mouse event
	void ProcessMouseEventL(struct vncmsg& aMsg);
	
	// Process a key event
	void ProcessKeyEventL(struct vncmsg& aMsg);

	// Process a clipboard copy
	void ProcessCopyClipboardToServerL(struct vncmsg& aMsg);

	// Process a clipboard copy
	void ProcessCopyClipboardFromServerL(struct vncmsg& aMsg);

	// Process a refresh
	void ProcessRefreshL(struct vncmsg& aMsg);

	// Process a foreground request
	void ProcessForegroundL(struct vncmsg& aMsg);

	// Process a quit request
	void ProcessQuitL(struct vncmsg& aMsg);

	// Find an action from the list of actions
	const XtActionsRec& FindActionL(const TDesC8& aAction);

	// Translate a unicode character to a key symbol
	static TInt TranslateUnicodeToKeysym(TInt aUnicode);
	
private:
	RWsSession 					iWsSession;			// The window server session
	RFs							iFs;				// The file server session
	CActiveScheduler*			iAs;				// The active scheduler if needed
	COptionParser* 				iOptionParser;		// Parse the options
	struct _WidgetRec			iDesktopWidget;		// The desktop widget
	Display						iDisplay;			// The current display mode
	Screen						iScreen;			// The current screen parameters
	Visual						iVisual;			// The visual data
	CPalette*					iPalette;			// The system palette for 256 colors
	TBuf8<15>					iGeometry;			// String of geometry
	RPointerArray<TAppInput>	iAppInputs;			// A list of app inputs
	RPointerArray<TAppEvent>	iAppEvents;			// A list of app events
	RArray<XtActionsRec>		iActions;			// A list of actions
	TDisplayMode				iDisplayMode;		// The selected display mode
	TRgb						iForegroundColor;	// The current foreground color

	// The main bitmap and bitmap manipulation	
	CFbsBitmap*					iMainBitmap;		// THE bitmap to store screen data
	CFbsBitmapDevice*			iBitmapDevice;		// Bitmap device to manipulate the bitmap
	CFbsBitGc*					iBitGc;				// Bitmap graphics context

	// Clipboard data
	char*						iClipboardText;		// Stored Clipboard text
	TInt						iClipboardLength;	// The current clipboard length

	// Jpeg manipulation
	CFbsBitmap*					iJpegBitmap;		// The current jpeg bitmap
	char*						iJpegData;			// Temporary data from the jpeg
	TInt						iDataStride;		// The width of the data for a line
			
	// Track allocated memory, so we can de-allocate it later
	RPointerArray<TAny>			iAllocatedMemory;	// Allocated generial memory
	RPointerArray<CPalette>		iAllocatedPalettes;	// Allocated palettes
	RPointerArray<CFbsBitmap>	iBitmaps;			// Allocated bitmaps

	// IPC mechanism
	CVncIpc*					iVncIpc;			// IPC mechanism
	};

// External access to the window system
extern CSymbianWindowSystem* WindowSystem();

// Externally destroy the windows system
extern void DestroyWindowSystem();

#endif // __X11Windows_H__
